package jsortdemo.algos;

/************************************************************************
  			HeapSort.java - Copyright 
**************************************************************************/

import jsortdemo.core.SortAlgo;

/**
 * Class HeapSort.
 *
 * <p>Project-Title: SortDemo --> Sortieralgorithmus HeapSort</p>
 
 * <p>Description: HeapSort ist ein Sortieralgorithmus des 
 * SortDemo-Projektes.
 * [dt. Sortierung von Haufen], ein 1964 von James W. J. Williams entwickelter Algorithmus,
 * mit dem beliebige Daten in auf- oder absteigender Reihenfolge sortiert werden. 
 * Er basiert auf dem 1962 von Robert W. Floyd verffentlichten Treesort-Verfahren. 
 * Fr die Heapsort-Sortierung werden die Daten zun�hst in einem binaeren Baum dargestellt,
 * dem Heap-Baum. Ein Heap-Baum ist binaer und ausserdem vollstaendig, d. h., 
 * er enthaelt mit Ausnahme der untersten Ebene keine Luecke. 
 * Zus�zlich soll der Baum noch geordnet sein, was allerdings erst durch die 
 * Sortierung gew�rleistet wird.
 * Beginnend von unten nach oben und von links nach rechts werden die Daten
 * paarweise so vertauscht, dass jedes an einem Knoten liegende Element groesser 
 * ist als die beiden Elemente in den dort entspringenden Aesten. Sobald das groesste
 * Element an der Spitze des Baums steht, wird es aus dem Baum entfernt und durch 
 * das am weitesten rechts unten stehenden Element ersetzt. Das aussortierte 
 * Element wird in die (sortierte) Ergebnisliste eingetragen.
 * Die fr eine Sortierung bentigte Zeit h�gt wie n �log (n) von der Anzahl n 
 * der zu sortierenden Elemente ab. Damit gehrt das Heapsort-Verfahren 
 * zu den schnellsten Sortieralgorithmen.</p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @see SortAlgo
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */
public class HeapSort extends SortAlgo{

private static int n;

/**
 * Initialisiert die Laenge und ruft heapsort() zum Sortieren auf
 * @throws InterruptedException
 */
protected final void sort() throws InterruptedException{
	n=len;
	heapsort();
}

/**
 * Hilfsfunktion fuer den HeapSort-Algorithmus.
 * @throws InterruptedException
 */
private final void heapsort() throws InterruptedException{
	buildheap();
	while (n>1){
		n--;
		swap (0, n);
		downheap (0);
	} 
}


/**
 * Hilfsfunktion fuer den HeapSort-Algorithmus-
 * teilt das Array in 2 Haelften und laesst diese Sortieren.
 */
protected final void buildheap() throws InterruptedException {
	for (int v=(n/2)-1; v>=0; v--)
		downheap (v);
}


/**
 * Hilfsfunktion fuer den HeapSort-Algorithmus-
 * uebernimmt das eigentliche Sortieren der Haufen.
 *
 *@param v Aktueller Wert von dem runtergezaehlt werden soll
 */
protected final void downheap(int v) throws InterruptedException{
	int w=2*v+1;    // first descendant of v
	while (w<n) {
		if (w+1<n)    // is there a second descendant?
		if ( comp(w+1, w) > 0 ) w++;
		// w is the descendant of v with maximum label
		if ( comp(v, w) >= 0) return;  // v has heap property
		// otherwise
		swap(v, w);  // exchange labels of v and w
		v=w;        // continue
		w=2*v+1;
	}
}

} // end of class
