package jsortdemo.algos;

import jsortdemo.core.SortAlgo;

/**
 * Class MergeSort.
 *
 * <p>Project-Title: SortDemo --> Sortieralgorithmus HeapSort</p>
 
 * <p>Description: MergeSort ist ein Sortieralgorithmus des 
 * SortDemo-Projektes. 
 * MergeSort funktioniert genauso wie Quicksort nach dem "divide-and-conquer"-Verfahren
 * Mergesort teilt das zu sortierende Array in zwei gleichgrosse Teilfolgen (Unterschied hoechstens
 * eine Komponente), sortiert diese (durch rekursive Anwendung von Mergesort auf die beiden
 * Teile) und mischt die dann sortierten Teile zusammen.
 * Vom MergeSort gibt es unterschiedliche Varianten.
 * Der normale MergeSort ist die Original-Methode, alle weiteren sind Abwandlungen
 * Beim Natural MergeSort werden z.B. natuerlich sortierte Folgen gesucht.
 * Dadurch werden Vergleichsoperationen eingespart.</p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @see SortAlgo
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */

public class MergeSort extends SortAlgo {

private static int[] b; // Hilfsarray b
private static int n;

/**
 * Initialisiert die Laenge, das tmp-Array und die gewaehlte Sortier-Merge-Methode
 * @throws InterruptedException
 */
protected final void sort() throws InterruptedException{
	n=len;
	b = new int[n];
	//initialisieren der gewaehlten Variante
	initMergeMethod();
}


/**
 * Sortiert das Array MergeSort (normal)
 * @throws InterruptedException
 * @param l Der kleinste Wert des Arrays
 * @param r Der groesste Wert des Arrays
 */
protected final void mergesort1(int l, int r) throws InterruptedException{
	if (l < r){
		int m = (l+r)/2;
		mergesort1(l, m);
		mergesort1(m+1, r);
		merge(l, m, r);
	}
}


/**
 * Uebernimmt fuer alle Merge-Methoden das eigentliche Sortieren/Werteaustauschen
 * @throws InterruptedException
 * @param l Der kleinste Index des Arrays
 * @param r Der groesste Index des Arrays
 * @param m Der mittlere Index des Arrays
 */
protected final void merge(int l, int m, int r) throws InterruptedException{

	int i, j, k;

	i =l;
	j = m+1;
	k = l;

	// vordere Haelfte von a in Hilfsarray b kopieren
	while ( (i <= m) &&  (j <= r) ){
		// minimum werte vom anfang suchen
		if ( comp(i,j) <= 0){
			b[k] = get(i);
			i++;
		}else{
			b[k] = get(j);
			j++;
		}
		k++;
	}

	if (i > m){
		// 1. Teil leer, rest des 2. teils kopieren
		for (int h = j; h <= r; h++){
			b[k+h-j] = get(h);
		}
	}else{
		for (int h = i; h <= m; h++){
			b[k+h-i] = get(h);
		}
	}

	// alles ins array schreiben
	for (int h = l; h <= r; h++){
		put(h, b[h]);
	}

}




// BEGIN METHODS STRAIGHT MERGESORT
/**
 * Sortiert das Array per straightMergeSort
 * @throws InterruptedException
 */
protected final void straightMergeSort() throws InterruptedException{
	int l = 0;
	int r = n-1;
	int size = 1;

	while (size < r-l+1){
		int rr = l -1;
		while ( (rr+size) < r ){
			int ll = rr +1;
			int mm = ll + size -1;
			if ((mm + size) <= r){
				rr = mm + size;
			}else{
				rr = r;
			}

			merge(ll,mm,rr);
		}
		size += size;
	}

}
// END METHODS STRAIGHT MERGESORT


// BEGIN METHODS NATURAL MERGESORT
/**
 * Sortiert das Array per NaturalMergeSort
 * @throws InterruptedException
 */
protected final void naturalMergeSort() throws InterruptedException{
	int l = 0;
	int r = n-1;
	int ll = r+1;

	while ( ll != l){
		int rr = l -1;
		while (rr != r){
			ll = rr +1;

			int mm = ll;
			while ( mm < r && (comp(mm+1, mm) >=0) ){
				mm++;
			}

			if  (mm < r ){
				rr = mm + 1;
				while ( rr < r && (comp(rr+1, rr) >= 0)){
					rr++;
				}
				merge(ll, mm, rr);
			} else {
			rr = mm;
			}
		}
	}
}
// END METHODS NATURAL MERGESORT

// Auswahl der MergeMethode
/**
 * Ruft je nach String die gewaehlte MergeSort Methode auf.
 * @throws InterruptedException
 */
protected final void initMergeMethod() throws InterruptedException{

	String mergeMethod;
	mergeMethod = getSecondArg();

	// Auswahl der gewaehlten Quicksortmethode
	if (mergeMethod.equals("normal")) {
		mergesort1(0, n-1);;
	} else if (mergeMethod.equals("straight")) {
		straightMergeSort();
	} else if (mergeMethod.equals("natural")){
		b=new int[n];
		naturalMergeSort();
	}
}

}    // end class MergeSort
