package jsortdemo.algos;

/************************************************************************
  			StraightSelection.java - Copyright 
**************************************************************************/

import jsortdemo.core.SortAlgo;

/**
 * Class MinMaxSelection.
 * <p>Project-Title: SortDemo --> Sortieralgorithmus MinMaxSelection</p>
 *
 * <p>Description: MinMaxSelection ist ein Sortieralgorithmus des 
 * SortDemo-Projektes.
 * SelectionSort (von engl. selection - Auswahl), auch MinSort (von Minimum) genannt,
 * ist ein naiver Sortieralgorithmus, der nicht stabil und in-place arbeitet. 
 * Die Komplexitaet von SelectionSort ist, in der Landau-Notation ausgedrueckt,
 * O(n2).
 * Um ein Array der Laenge n zu sortieren, wird das Minimum gesucht, also das Element
 * mit dem kleinsten Sortierschluessel. Das Minimum wird dann mit dem ersten Element
 * des Arrays vertauscht. Somit erhoeht man links ein sortiertes 
 * Teilarray der Laenge 1 und rechts ein unsortiertes der Laenge n-1. Anschliessend
 * wird der Algorithmus auf das unsortierte Teilarray angewendet.
 * Bei der MinMaxSelection wird paralell minimum und maximum gesucht. 
 * </p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @see SortAlgo
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */ 
public class MinMaxSelection extends SortAlgo {

static int n; 
	
/**
 * Sortiert das Array per Straight Selection
 * @throws InterruptedException
 */
protected final void sort() throws InterruptedException {
// variant of StraightSelection
// minimum and maximum is computed in parallel
// like BubbleSort and ShakerSort

	n = len;

	for (int i=0, j=len-1; i<j; i++, j--) {
		
		int minix = i;
		int maxix = i;
		int min = get(minix);
		int max = min;
		
		for (int k = i+1; k <= j; k++) {

	    		if ( comp(k, minix) < 0) {
				minix = k;
				min = get(minix);
	    		} else {
				if ( comp(maxix, k)< 0 ) {
		    			maxix = k;
		    			max = get(k);
				}
	    		}
		}

	put(minix,get(i));
	put(i,min);

	if (maxix==i) maxix=minix;

	put(maxix, get(j) );
	put(j, max);
    }
}

}
