package jsortdemo.algos;

/************************************************************************
  			QuickSort.java - Copyright 
**************************************************************************/

import jsortdemo.core.SortAlgo;
import java.util.Random;

/**
 * Class QuickSort.
 *
 * <p>Project-Title: SortDemo --> Sortieralgorithmus QuickSort</p>
 *
 * <p>Description: QuickSort ist ein Sortieralgorithmus des 
 * SortDemo-Projektes.
 * Quicksort basiert (im Gegensatz zu Mergesort) auf variabler Aufteilung des Eingabearrays.
 * Es wurde 1962 von Hoare entwickelt. 
 * Der Quicksort-Algorithmus ist eines der schnellsten und zugleich einfachsten Sortierverfahren.
 * Das Verfahren arbeitet rekursiv nach dem Divide-and-Conquer-Prinzip und benoetigt im Worst Case 
 * (n2) Vergleiche, im Mittel jedoch nur O(n log n) Vergleiche, und ist aufgrund
 * empirischer Vergleiche allen anderen O(n log n) Sortierverfahren ueberlegen.
 * Die 3 Varianten (first, middle, random) unterscheiden sich durch die unterschiedlichen
 * Anfangswerte - ebene first, middle oder random --> eins von beiden</p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @see SortAlgo
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */ 
public class QuickSort extends SortAlgo {


/**
 * Sortiert das Array per QuickSort
 * @throws InterruptedException
 */
protected final void sort() throws InterruptedException {
	quicksort(0, len-1);
}


/**
 * Waehlt je nach selektierter Methode den entsprechenden Startwert der Sortierung
 *
 * @return gibt den Startwert fuer die Sortierung zurueck
 * @throws InterruptedException
 * @param lo Der Index des ersten Wertes im Array
 * @param hi Der Index des letzten Wertes im Array
 */
protected final int getMethodVal(int lo, int hi) throws InterruptedException{
	
	String quicksortMethod;
	quicksortMethod = getSecondArg();
	
	// Auswahl der gewaehlten Quicksortmethode
	if (quicksortMethod.equals("first")) {
		return get(lo);
	} else if (quicksortMethod.equals("middle")) {
		return get((lo+hi)/2);
	} else if (quicksortMethod.equals("random")){
		Random rgen = new Random();  // Random number generator
		int rand = rgen.nextInt(hi-lo+1);
		return get(rand + lo);
	} else return 0;
}


/**
 * Sortiert das Array per QuickSort
 * @throws InterruptedException
 * @param lo Der Index des ersten Wertes im Array
 * @param hi Der Index des letzten Wertes im Array
 */
protected final void quicksort(int lo, int hi) throws InterruptedException{
	
	int i=lo, j=hi, x;
	// Startwert holen
	x = getMethodVal(lo,hi);
	//  Aufteilung
	while (i<=j){    
		while (compVal( get(i), x ) < 0)i++;
		while (compVal( get(j), x ) > 0)j--;
		if (i<=j){
			swap(i, j);
			i++; j--;
		}
	}
	
	// Rekursion
	if (lo<j) quicksort(lo, j);
	if (i<hi) quicksort(i, hi);
}

}    // end class QuickSort

