package jsortdemo.core;

/************************************************************************
  			MainApplet.java - Copyright 
**************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;
import java.lang.reflect.InvocationTargetException;


/**
 * Class MainApplet.
 * <p>Project-Title: SortDemo --> MainApplet</p>
 *
 * <p>Description: Das im Rahmen des Softwarepraktikums im WS04/05 entstandene 
 * Programm SortDemo visualisiert verschiedene Sortieralgorithmen.
 * Als Grundlage dient ein unsortiertes Array aus Integerwerten, die jeweils nur 
 * ein einziges Mal auftreten. 
 * Um die Effizienz der einzelnen Algorithmen zu verdeutlichen, werden jeweils
 * die writes, reads, compares und swaps gezaehlt. 
 * Neben 14 unterschiedliche Algos besteht die Moeglichkeit, die Groesse des 
 * Arrays zwischen 10 und 1000 zu variieren, das Array genauso wie das vorherige
 * zu sortieren oder die Visualisierung schneller bzw. langsamer abspielen zu 
 * lassen
 * </p>
 *
 * <p>Klasse MainApplet: Zustaendig fuer den Aufbau des Applets, das initialisieren
 * aller Applet- und Modell-Komponenten und das Zusammenspiel zwischen Interaktion
 * des Benutzers und Reaktion 
 * </p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */
public class MainApplet extends JApplet implements ActionListener {


////////////////////////////////////////////////////////////////////////
//                        Objectreferences
////////////////////////////////////////////////////////////////////////

SortModel sortModel; 
ArrayView arrayView; 
ParamView paramView; 
JScrollPane scrollPane;
DelayPanel delayPanel; 
Menu bar;

ActionListener algoListener;
ActionListener countListener;
ActionListener viewListener;
ChangeListener delaySliderListener;
MouseAdapter mouseAdapter;
KeyAdapter keyAdapter;

/*
 * Standarmaessig stellt das Applet den 'rootContainer'
 * und diese Variable bleibt wie sie ist.
 *
 * Laeuft es allerdings im Standalone-Modus, wird diese
 * Referenz in initStandalone auf die ContentPane des 
 * umgebenden JFrame gesetzt.
 *
 * 'rootContainer' dient der Funktion isStandalone() als
 * eindeutiger Indikator, sowie ggf. Popup-Dialogen als
 * Parent.
 */
Container rootContainer = getContentPane();



////////////////////////////////////////////////////////////////////////
//                           Constants
////////////////////////////////////////////////////////////////////////

// Array fuer die Array-Groessen
static final int[] ARRAY_SIZES = { 10, 20, 40, 80, 100, 200, 
				   300, 400, 500, 1000 };

// String-Array mit allen Namen der Algos
static final String[] ALGO_NAMES = { 
	"StraightInsertion", 
	"BinaryInsertion", 
	"StraightSelection", 
	"MinMaxSelection", 
	"BubbleSort", 
	"ShakerSort", 
	"ShellSort", 
	"HeapSort", 
	"QuickSort (first)", 
	"QuickSort (middle)", 
	"QuickSort (random)", 
	"MergeSort (normal)",
	"MergeSort (straight)", 
	"MergeSort (natural)" 
};

/*
 * Kommandozeilen- oder Appletparameter
 * Name, Beschreibung
 */
static final String[][] PARAMS = { 
	{"width", 
	 "Initial horizontal window size in pixels (standalone only)" },

	{"height", 
	 "Initial vertical window size in pixels (standalone only)" },
	
	{"algoNameIndex", 
	 "Index of the initial sorting algorithm" }, 

	{"arraySizeIndex", 
	 "Index of the initial number of elements in the array" },

	{"delay", 
	 "Initial value for delay per operation" },

	{"viewIndex", 
	 "Index of the initial viewing mode" }
};

/* 
 * Integerwerte fuer Kommandozeilen- oder Appletparameter
 * min, max, default
 */
static final int[][] PARAM_VALUES = {
	{ 1, Integer.MAX_VALUE, 	600 },
	{ 1, Integer.MAX_VALUE, 	320 },
	{ 0, ALGO_NAMES.length-1, 	2 },
	{ 0, ARRAY_SIZES.length-1, 	2 },
	{ 0, 500,		  	20 },
	{ 0, ArrayView.VIEWS.length-1,	1 }
};

/* es folgen die Info-Strings */

static final String ABOUT_INFO = 
	"This is jsortdemo, version 1.1\n"+
	"written (c) 2004 by\n"+
	"   Yvonne von Bischopinck\n"+
	"   Mathis Ahrens";

static final String SHORTCUTS_INFO = 
	"Start, Pause, Continue: \n"+
	"   hit <SPACE> or your right mouse button \n"+
	"\n"+
	"Delay: \n"+
	"   use the arrow keys (up/down)!\n"+
	"\n"+
	"Fill: \n"+
	"   hit 'f' or your middle mouse button\n"+
	"(this will use the selected fill Method to reset the field)\n";

static final String COUNTER_INFO =
	"Those Numbers at the bottom are...?\n"+
	"\n"+
	"Reads/Writes:\n"+
	"   Counting each and every read/write-operation that operates\n"+
	"   _on the field_ (as opposed to RAM). These should be a good\n"+
	"   general measure for the amount of disk access of an alogrithm.\n"+
	"\n"+
	"Swaps:\n"+
	"   Exchanging two values. This always also adds 2 reads + 2 writes.\n"+
	"\n"+
	"Compares:\n"+
	"   This depends: Usually it means that 2 reads add up, \n"+
	"   but compares in RAM get counted also - in that case no read\n"+
	"   shows up, because those values were obviously read before!\n";

static final String MORE_HELP =
	"Wha??\n"+
	"\n"+
	"Oh well, try 'docs/userdoc.txt' or 'docs/api-docs'\n"+
	"\n";
	

////////////////////////////////////////////////////////////////////////
//                           Methods
////////////////////////////////////////////////////////////////////////


/**
 * Gibt die AppletInfos zurueck
 * @return String die Infos
 */
public String getAppletInfo() {
	return ABOUT_INFO;
}


/**
 * Implementierung von Applet.getParameterInfo()
 *
 * Es wird ein String-Array mit allen moeglichen Parametern zusammengestellt,
 * die erkannt werden. Das Format ist gemaess Konvention:
 * Name, Typ+Wertebereich, Funktionsbeschreibung
 *
 *@return String[][] Die Parameter-Informationen
 */
public String[][] getParameterInfo() {


	String[][] res = new String[PARAMS.length][3];

	for ( int i=0; i<PARAMS.length; i++ ) {
		res[i][0] = PARAMS[i][0];
		res[i][1] = "Integer, " +   PARAM_VALUES[i][0] +
			    " - " +         PARAM_VALUES[i][1] +
			    ", default: " + PARAM_VALUES[i][2];
		res[i][2] = PARAMS[i][1];
	}

	return res;

}

/**
 * Variante von getParameterInfo(), die einen einzigen String ausgibt.
 * (nuetzlich fuer JavaScript-alert)
 *
 *@return String Einzelne Parameter-Info
 */
public String getParamInfo() {

	StringBuffer res = new StringBuffer();
	String[][] pi = getParameterInfo();

	for ( int i=0; i<PARAMS.length; i++ )
	{
//		res.append("-------------------\n");

		for ( int j=0; j<3; j++ )
			res.append( pi[i][j] + "\n" );

		res.append("\n");
	}
//	res.append("-------------------\n");

	return res.toString();
}

/**
 * Liefert den Index im PARAM-Array zu einem Parameternamen 
 *
 * @param key Der Schluessel, dessen Index gesucht wird.
 * @return int Bei Erfolg der Index, sonst -1
 */
private int getParamIndex(String key) {

	for (int i=0; i<PARAMS.length; i++)
	{
		if (key.equals(PARAMS[i][0]))
			return i;
	}
	// should never happen
	return -1;
}

/**
 * Gibt den (int-)Wert zurueck, der fuer einen Parameter benutzt werden soll.
 * Das ist der Standardwert, wenn kein solcher Kommandozeilen- oder Appletparameter
 * angegeben wurde oder dieser nicht geparst werden kann.
 * Ist eine Angabe vorhanden und gueltig, so wird sie uebernommen, ggf in das Interval
 * [Minimum,Maximum] verschoben.
 *
 * @param key Der Schluessel, dessen Wert ausgelesen/festgestellt werden soll.
 * @return int Der ermittlete Wert
 */
private int getIntParam(String key) {

	Integer parsedInt;

	// Parameter erstmal auslesen
	if ( isStandalone() )
		parsedInt = Integer.getInteger(key);
	else
		// Applet-Mode
		try {
			parsedInt = Integer.valueOf( getParameter(key) );
		} catch ( NumberFormatException e ) {
			parsedInt = null;
		}

	
	int index = getParamIndex(key);
	int min = PARAM_VALUES[index][0];
	int max = PARAM_VALUES[index][1];
	int def = PARAM_VALUES[index][2]; //default

	// Konsistenzchecks
	assert ((min<=max) && (min<=def) && (def<=max)) :
		"PARAM_VALUES ist inkonsistent (index: "+index+").";


	// Wenn kein Parameter geparst werden konnte, Defaultwert zurueckgeben
	if (parsedInt==null)
	{
		System.out.println("Info: Kein gueltiger Parameter '"+key+
				   "' -> wir nehmen den Standardwert.");
		return def;
	}
	else
	{
		// sonst ausgelesenen Wert gegen min,max testen und zurueckgeben
		int pint = parsedInt.intValue();

		if (pint>max)
			return max;
		else if (pint<min)
			return min;
		else
			return pint;
	}

}


/**
 * Ueberpruefung ob das Applet im Standalone-modus laeuft oder nicht
 * 
 * @return boolean True wenn standalone sonst false
 */
private boolean isStandalone() 
{ 
	// laueft diese Klasse Standalone, so ist
	// die ContentPane des _JFrames_ rootContainer 
	return (rootContainer!=getContentPane()); 
}


/**
 * Initialisierung des Standalone-Modus
 * @param frame Frame der den RootContainer fuer das Applet erstellt
 */
public void initStandalone( JFrame frame ) {

    rootContainer = frame.getContentPane();
    frame.setSize( new Dimension(getIntParam("width"),getIntParam("height")));
    init();

}


/**
 * Initialisieren des Applets und dessen Komponenten auf Basis der gesetzten Params
 */
public void init() {

	assert (PARAM_VALUES.length == PARAMS.length) :
	"PARAM_VALUES und PARAMS haben unterschiedliche Laenge!";

	// bequemlichkeitsvariablen
	int algoNameIndex = getIntParam("algoNameIndex");
	int arraySizeIndex = getIntParam("arraySizeIndex");
	int viewIndex = getIntParam("viewIndex");
	int initialDelay = getIntParam("delay");
	int delayMinMax[] = PARAM_VALUES[ getParamIndex("delay") ];
	
	// Alle anderen noetigen Objekte erzeugen
	sortModel = new SortModel( ALGO_NAMES[algoNameIndex],
			           ARRAY_SIZES[arraySizeIndex],
			           initialDelay );
	arrayView = new ArrayView( sortModel, viewIndex );
	paramView = new ParamView( sortModel );
	scrollPane = new JScrollPane( arrayView );
	delayPanel = new DelayPanel(initialDelay, delayMinMax);
	bar = new Menu( isStandalone() );

	// JMenuBar: Jeweils einen RadioButton markieren
	bar.jRadioAlgos[algoNameIndex].setSelected(true);
	bar.jRadioArraySizes[arraySizeIndex].setSelected(true);
	bar.jRadioViews[viewIndex].setSelected(true);

	// Action-, Change- und MouseListener initialisieren
	initListeners();

	// alles hinzufuegen
	setJMenuBar( bar );
	getContentPane().add(scrollPane, BorderLayout.CENTER);
	getContentPane().add(paramView, BorderLayout.SOUTH);
	getContentPane().add(delayPanel, BorderLayout.WEST);


}


/**
 * Initialisieren von ActionListenern und Hinzufuegen zu allen
 * Applet-Komponenten, um Aenderungen feststellen zu koennen.
 */
private void initListeners() {

	// einzelner Listener fuer die Algorithmen
	algoListener = new ActionListener() { 
		public void actionPerformed( ActionEvent e ) {
			sortModel.setAlgo(((JRadioButtonMenuItem)(e.getSource())).getText());
			arrayView.update(null,null);
		}
	};
	
	// Listener fuer die Anzahl der Werte im Array
	countListener = new ActionListener() { 
		public void actionPerformed( ActionEvent e ) {
			sortModel.resetSortAlgo();
			String s = ((JRadioButtonMenuItem)(e.getSource())).getText();
			sortModel.resize(Integer.parseInt(s));
			fillArrayWithSelectedMethod();
		}
	};
	
	// Listener fuer die Anzahl der Werte im Array
	viewListener = new ActionListener() { 
		public void actionPerformed( ActionEvent e ) {
			String s = ((JRadioButtonMenuItem)(e.getSource())).getText();
			arrayView.setView(s);
		}
	};

	// Listener fuer den Geschwindigkeitsslider
	delaySliderListener = new ChangeListener() {
		public void stateChanged( ChangeEvent e ) {
			sortModel.delay = ((JSlider)(e.getSource())).getValue();
			delayPanel.setLabelText(sortModel.delay);
		} 
	};

	// Listener um auf MouseEvents reagieren zu koennen
	mouseAdapter = new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
			arrayView.requestFocusInWindow();
			if ( e.getButton() == MouseEvent.BUTTON3 )
				processStartStopContinueEvent();
			else if ( e.getButton() == MouseEvent.BUTTON2 )
				processFillEvent();
		}
	};

	// Listener fuer KeyPress Events
	// work around slow accelerator/mnemonic for Start/Stop:
	// let arrayView's KeyListener catch <SPACE>
	keyAdapter = new KeyAdapter() {
		public void keyPressed(KeyEvent e) {
			switch ( e.getKeyCode() )
			{
				case KeyEvent.VK_SPACE:
					processStartStopContinueEvent();
					break;

				case KeyEvent.VK_UP:
					delayPanel.slider.setValue( delayPanel.slider.getValue() + 5 );
					break;

				case KeyEvent.VK_DOWN:
					delayPanel.slider.setValue( delayPanel.slider.getValue() - 5 );
					break;
			}
		}
	};


	// alle Listener hinzufuegen
	delayPanel.slider.addChangeListener( delaySliderListener );
	sortModel.addObserver( bar );
	arrayView.addMouseListener( mouseAdapter );

	delayPanel.slider.setFocusable(false);
	arrayView.addKeyListener( keyAdapter );
	

	bar.jMenuItemFillNow.addActionListener(this);
	bar.jMenuItemStartStop.addActionListener(this);
	//bar.jMenuItemCancel.addActionListener(this);
	bar.jMenuItemExit.addActionListener(this);
	bar.jMenuItemCounting.addActionListener(this);
	bar.jMenuItemShortcuts.addActionListener(this);
	bar.jMenuItemParams.addActionListener(this);
	bar.jMenuItemAbout.addActionListener(this);
	bar.jMenuItemMoreHelp.addActionListener(this);

	for (int i=0; i<ALGO_NAMES.length ; i++)
		bar.jRadioAlgos[i].addActionListener( algoListener );
	
	for (int j=0; j<ARRAY_SIZES.length ; j++) 
		bar.jRadioArraySizes[j].addActionListener( countListener );

	for (int k=0; k<ArrayView.VIEWS.length ; k++) 
		bar.jRadioViews[k].addActionListener( viewListener );
} 



/**
 * Fuellt das Array auf die vom Benutzer in Array-Item selektierte Methode
 */
private void fillArrayWithSelectedMethod()
{
	if ( bar.jRadioPreviousRandom.isSelected() )
		sortModel.setArrayToPreviousRandom();

	else if ( bar.jRadioRandom.isSelected() )
		sortModel.setArrayToRandom();

	else if ( bar.jRadioReverseSorted.isSelected() )
		sortModel.setArrayToReverseSorted();

	else if ( bar.jRadioSorted.isSelected() )
		sortModel.setArrayToSorted();
}


/**
 * Reaktion auf rechten Maustastendruck oder SPACE;
 * Je nach aktuellem Zusatand wird das Sortieren gestoppt,gestartet oder pausiert
 */
private void processStartStopContinueEvent()
{
	if ( ! sortModel.isAlive() )
	{
		// start new Sort
		sortModel.startSortAlgo();
		bar.jMenuItemStartStop.setText("Pause");
	}
	else if ( sortModel.paused )
	{
		// resume from pause
		sortModel.setPaused( false );
		bar.jMenuItemStartStop.setText("Pause");
	}
	else
	{
		// pause
		sortModel.setPaused( true );
		bar.jMenuItemStartStop.setText("Continue");
	}
}


/**
 * Reaktion auf mittleren Maustastendruck oder 'f';
 * Das Array wird mit neuen Werten gefuellt und der Algo zurueckgesetzt
 */
private void processFillEvent()
{
		sortModel.resetSortAlgo();
		fillArrayWithSelectedMethod();
}


/**
 * Es wird ein Dialog angezeigt, der seinen Inhalt in einem resize-
 * und scrollfaehigen Frame darstellt.
 *
 * @param content der anzuzeigende Text
 * @param title der Titel in der Frameleiste
 *
 */
private void showInfoDialog( String content, String title )
{
	JOptionPane jop = new JOptionPane( content, JOptionPane.PLAIN_MESSAGE ); 
	JInternalFrame jif = jop.createInternalFrame( rootContainer, title );
	Container cp = jif.getContentPane();

	//Scrollpane hinzufuegen (Evil Hack#1)
	cp.remove( jop );
	JScrollPane jsp = new JScrollPane( jop );
	cp.add( jsp, BorderLayout.CENTER );

	jif.setClosable(true);
	jif.setResizable(true);

	// Groesse und Position anpassen
	Dimension prefDim = jif.getPreferredSize();
	Dimension rootDim = rootContainer.getSize();

	// Evil Hack#2
	prefDim.width = prefDim.width+15; 

	// Dialog maximal Fenstergroesse
	if (prefDim.width>rootDim.width)
		prefDim.width = rootDim.width;
	if (prefDim.height>rootDim.height)
		prefDim.height = rootDim.height;

	jif.setSize(prefDim);

	jif.show();
}


/* ************************************************************************
 * ACTION LISTENING
 * ************************************************************************/

/**
 * Sobald der Benutzer eine Interaktion startet, welche Funktion darauf zu
 *reagieren hat
 * @param e  Welches Event hat stattgefunden/was wurde angeklickt?
 */
public void actionPerformed( ActionEvent e ) {


	Object source = e.getSource();

	// FillNow
	if (source == bar.jMenuItemFillNow)
		processFillEvent();

	// StartStopContinue
	else if (source == bar.jMenuItemStartStop)
	{
		System.out.println("x");
		processStartStopContinueEvent();
	}
	
	// CancelSort
	// !! deleted - replaced by pause !!
	/*
	else if ( source == bar.jMenuItemCancel )
	{
		sortModel.resetSortAlgo();
		bar.jMenuItemStartStop.setText("Start");
	}
	*/

	// Exit
	else if ( source == bar.jMenuItemExit )
		System.exit(0);

	// Info: Counting
	else if (source == bar.jMenuItemCounting)
		showInfoDialog( COUNTER_INFO, 
				bar.jMenuItemCounting.getText() );
	// Info: Shortcuts
	else if (source == bar.jMenuItemShortcuts)
		showInfoDialog( SHORTCUTS_INFO, 
				bar.jMenuItemShortcuts.getText() );
	// Info: Params
	else if (source == bar.jMenuItemParams)
		showInfoDialog( getParamInfo(), 
				bar.jMenuItemParams.getText());
	// Info: About
	else if (source == bar.jMenuItemAbout)
		showInfoDialog( ABOUT_INFO, 
				bar.jMenuItemAbout.getText());
	// Info: MoreHelp
	else if (source == bar.jMenuItemMoreHelp)
		showInfoDialog( MORE_HELP, 
				bar.jMenuItemMoreHelp.getText());
		
	

	
}


/* ************************************************************************
 * STANDALONE-MODE FUNCTIONS
 * ************************************************************************/

/**
 * initStandaloneGUI:
 * baut die GUI fuer den Standalone-Betrieb im 
 * event-dispatching-Thread auf
 *
 * @param applet Uebergibt das komplette Applet
 **/
private static void 
initStandaloneGUI( final MainApplet applet ) throws Exception {

	// 'invokeAndWait' statt 'invokeLater', damit das Applet 
	// nicht gestartet wird, bevor dieser Code komplett 
	// ausgefuehrt ist

	try {
	javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
		public void run() {

			final JFrame frame = new JFrame("jSortDemo (Standalone Mode)") ;

			frame.addComponentListener( new ComponentAdapter() {
				public void componentResized(ComponentEvent e) {
					applet.arrayView.update( null, null );
				}
			}
			);
	
			// Programm beenden, wenn auf [X] geklickt wird
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			// Optional: Specify who draws the window decorations. 
			JFrame.setDefaultLookAndFeelDecorated(true);

			applet.initStandalone( frame );
			
			// ContentPane und JMenuBar werden einzeln auf die des
			// Applets gesetzt, da 
			//   frame.getContentPane().add( applet );
			// das Double-Buffering durcheinander bringt :-(

			frame.getContentPane().add( applet.getContentPane() );
			frame.setJMenuBar( applet.getJMenuBar() );
			
			// Focus sollte immer beim View bleiben	 
			frame.setFocusable(false);
			applet.setFocusable(false);

			frame.setLocation(100,100);
			frame.setVisible(true);
			
		}
	});
	} catch (InvocationTargetException ite)  {
		ite.printStackTrace();	
		System.out.println("Cause: "+ite.getCause());
	}

}


/**
 * Main-Method, falls das Applet im Standalone-Betrieb gestartet werden soll
 *
 * @param args Uebergibt die Kommandozeilenparameter
 **/
public static void main(String[] args) {

	final MainApplet applet = new MainApplet();

	// den Aufbau der GUI Thread-sicher 
	// im event-dispatching-Thread ausfuehren.
	try {
		initStandaloneGUI( applet );
	}
	catch (Exception e) {
		System.out.println("Fehler beim Initialisieren der GUI:\n");
		e.printStackTrace();
		System.out.println("\nProgramm wird beendet...\n");
		System.exit(0);
	}

	// work around slow accelerator/mnemonic for Start/Stop:
	// let arrayView's KeyListener catch <SPACE>
	// and give him focus immediately after creation
	applet.arrayView.requestFocusInWindow();


}
}

