package jsortdemo.core;


/************************************************************************
  			Menu.java - Copyright 
**************************************************************************/
/**
 * Class Menu.
 *
 * <p>Project-Title: SortDemo --> Klasse um das Menu im Applet zu bauen</p>
 *
 * <p>Description: Die Klasse Menu ist fr alle Interaktionen des Menus im 
 * Applet zustaendig. Um auf Aenderungen reagieren zu koennen, 
 * implementiert sie die Java-Klasse Observer</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: </p>
 *
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */ 


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public class Menu extends JMenuBar implements Observer {

// Variablendeklaration fuer alle Bestandteile die im Menu gebraucht werden
JMenu jMenuRun = new JMenu("Run");
 JMenuItem jMenuItemStartStop = new JMenuItem("Start");
 //JMenuItem jMenuItemCancel = new JMenuItem("Cancel");
 JMenuItem jMenuItemExit = new JMenuItem("Exit jsortdemo");

JMenu jMenuAlgorithm = new JMenu("Algorithm");
 JRadioButtonMenuItem jRadioAlgos[] = new JRadioButtonMenuItem[ MainApplet.ALGO_NAMES.length ];

JMenu jMenuFill = new JMenu("Fill");
 JMenuItem jMenuItemFillNow = new JMenuItem("Fill Now");
 JRadioButtonMenuItem jRadioRandom = new JRadioButtonMenuItem("Random Fill", true);
 JRadioButtonMenuItem jRadioSorted = new JRadioButtonMenuItem("Sorted Fill");
 JRadioButtonMenuItem jRadioReverseSorted = new JRadioButtonMenuItem("Reverse Sorted Fill");
 JRadioButtonMenuItem jRadioPreviousRandom = new JRadioButtonMenuItem("Previous Random Fill");

JMenu jMenuSize = new JMenu("Size");
 JRadioButtonMenuItem jRadioArraySizes[] = new JRadioButtonMenuItem[ MainApplet.ARRAY_SIZES.length ];

JMenu jMenuView = new JMenu("View");
 JRadioButtonMenuItem jRadioViews[] = new JRadioButtonMenuItem[ ArrayView.VIEWS.length ];

JMenu jMenuInfos = new JMenu("Info");
 JMenuItem jMenuItemCounting = new JMenuItem("Reads/Writes/Comps/Swaps");
 JMenuItem jMenuItemShortcuts = new JMenuItem("Keyboard&Mouse Shortcuts");
 JMenuItem jMenuItemParams = new JMenuItem("Startup Parameters");
 JMenuItem jMenuItemAbout = new JMenuItem("About / Authors");
 JMenuItem jMenuItemMoreHelp = new JMenuItem("Need more help...");

JLabel jLabelArrow = new JLabel(" ---> ");

ButtonGroup sequenceGroup = new ButtonGroup();
ButtonGroup algoGroup = new ButtonGroup();
ButtonGroup countGroup = new ButtonGroup();
ButtonGroup viewGroup = new ButtonGroup();


// Konstruktor
/**
 * Konstruktor der Menu-Klasse: Aufbau des Menus & setzen der Abkuerzungen zum 
 * Aufruf(Acceleratoren/Shortcuts)
 */
public Menu (boolean isStandalone) {

	buildMenu(isStandalone);
	setAccelerators();

}



/**
 * Hinzufuegen der MenuItems zur Menubar
 */
private void buildMenu(boolean isStandalone) {

	add( jLabelArrow );

	add(jMenuRun);
	 jMenuRun.add(jMenuItemStartStop);
	 //jMenuRun.add(jMenuItemCancel);
	 if (isStandalone)
	 {
		 jMenuRun.addSeparator();
		 jMenuRun.add(jMenuItemExit);
	 }

	add(jMenuAlgorithm);
	 for (int i=0; i<MainApplet.ALGO_NAMES.length ; i++) {
		jRadioAlgos[i] = new JRadioButtonMenuItem( MainApplet.ALGO_NAMES[i]);
		jMenuAlgorithm.add(jRadioAlgos[i]);
		algoGroup.add( jRadioAlgos[i] );
	 }

	add(jMenuFill);

	 jMenuFill.add(jMenuItemFillNow);
	 jMenuFill.addSeparator();

	 // Sequence ButtonGroup 
	 sequenceGroup.add(jRadioSorted);
	 sequenceGroup.add(jRadioReverseSorted);
	 sequenceGroup.add(jRadioRandom);
	 sequenceGroup.add(jRadioPreviousRandom);
 
	 jMenuFill.add(jRadioSorted);
	 jMenuFill.add(jRadioReverseSorted);
	 jMenuFill.add(jRadioRandom);
	 jMenuFill.add(jRadioPreviousRandom);
	 

	add(jMenuSize);
	 for (int j=0; j<MainApplet.ARRAY_SIZES.length ; j++) {
		jRadioArraySizes[j] = new JRadioButtonMenuItem( "" + 
				MainApplet.ARRAY_SIZES[j] );
		jMenuSize.add(jRadioArraySizes[j]);
		countGroup.add( jRadioArraySizes[j] );
	  }

	add(jMenuView);
	 for (int k=0; k<ArrayView.VIEWS.length ; k++) {
		jRadioViews[k] = new JRadioButtonMenuItem( "" + 
				ArrayView.VIEWS[k] );
		jMenuView.add(jRadioViews[k]);
		viewGroup.add(jRadioViews[k]);
	 }


	add(jMenuInfos);
	 jMenuInfos.add(jMenuItemCounting);
	 jMenuInfos.add(jMenuItemShortcuts);
	 jMenuInfos.add(jMenuItemParams);
	 jMenuInfos.add(jMenuItemAbout);
	 jMenuInfos.add(jMenuItemMoreHelp);
	
}


/**
 * Setzt die Tastenabkuerzungen fuer die Buttons, um oft benoetigte Buttons schneller 
 * benutzen zu koennen
 */
private void setAccelerators()
{
	 // Start/Pause/Continue
	 //!! Work around slow key binding by _not_ using accelerator or mnemonic,
	 //!! but a keyAdapter instead
	 //jMenuItemStartStop.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_0, 0 )); 

	 // Cancel
	 // !! Deleted - not needed anymore: 'pause' will do. !!
	 //jMenuItemCancel.setAccelerator(KeyStroke.getKeyStroke('c'));

	 // Exit
	 jMenuItemExit.setAccelerator(KeyStroke.getKeyStroke(
				 KeyEvent.VK_ESCAPE, 0));

	 // FillNow
	 jMenuItemFillNow.setAccelerator(KeyStroke.getKeyStroke('f'));

}




/**
 * ************************************************************************
 * OBSERVING CHANGES
 * ************************************************************************
 **/

/**
 * Setzt den Startbutton wieder zturueck auf Start, sobald das Modell
 * mit Sortieren fertig ist
 * @param o Das Objekt, nach dessen Ende alles upgedated werden soll
 * @param arg Von Observer vorgeschriebener Parameter, der hier aber nicht ausgelesen wird
 */
public void update( Observable o, Object arg ) {
	if (((SortModel)o).finished)	
	{
		jMenuItemStartStop.setText("Start");
	}
}

}
