package jsortdemo.core;

/************************************************************************
  			SortAlgo.java - Copyright 
**************************************************************************/


/**
 * Class SortAlgo.
 *
 * <p>Project-Title: SortDemo --> Abstrakte Klasse fuer Sortieralgorithmen</p>
 *
 * <p>Description: SortAlgo ist die abstrakte Klasse fuer alle Sortieralgorithmen
 * und stellt Zugriffsfunktionen und -Methoden zum Vergleichen und Austauschen der Array-Werte bereit</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */ 

public abstract class SortAlgo extends Thread {
  // Fields
  protected SortModel p;
  // Laenge des Arrays
  protected int len;

/**
 * Liefert den zweiten Teil eines Algorithmusnamens
 *
 * Bei Sortieralgorithmen, die aus mehreren Varianten bestehen, gibt der 2. Teilstring die entsprechende MEthode dar.
 * Dieser Teil wird von der Funktion getSecondArg abgeschnitten und als STring zurckgegeben
 *
 * @return String den zweiten Teil des Algonamens
 */
public String getSecondArg() {
  	// Leerzeichen als Wortseparator 
	
	String[] tmp = p.getAlgoName().split(" ");
	String res;
	res = tmp[1].substring(tmp[1].indexOf('(')+1,tmp[1].indexOf(')') ); 
	
	return res;	
  }

/**
 * Setzt das Parent-Model
 * @param parent Referenz auf das SortModell
 */
public void setParent( SortModel parent ) {
	 p = parent;
  }

/**
 * Versucht den Sortiervorgang zu starten
 */
public void run() {

	//System.out.println("...run()...");
	len = p.getLength();

	try
	{
		sort();
		System.out.println("sort() returned!");
	} 
	catch (InterruptedException e) 
	{
		System.out.println("Sort Interrupted!");
	}

	p.algoFinished();
  }

/**
 * Die Sortiermethode, die von den einzelnen Algos ueberschrieben wird
 */
abstract protected void sort() throws InterruptedException; 


/***********************************************************
 * Zugriffsfunktionen fuer Algorithmen
 **********************************************************/

/**
 * Liefert den Wert der am uebergebenen Index im Array steht
 *
 *@param index  Der Index der geliefert werden soll
 *@return int Gibt den Wert an Index i zurueck
 *@throws InterruptedException
 */
protected int get( int index ) throws InterruptedException {
	  p.currOp[0] = p.READ;
	  p.currOp[1] = index;
	  p.currOp[2] = -1;
	  return p.get(index);
  }

 
/**
 * Schreibt den uebergebenenen Wert im Array an die uebergebene Index-Stelle
 *
 *@param index Die Stelle an die der Wert geschrieben werden soll
 *@param  value Der Wert der an Index i geschrieben werden soll
 *@throws InterruptedException
 *@return void
 */
protected void put( int index, int value ) throws InterruptedException {
	  p.currOp[0] = p.WRITE;
	  p.currOp[1] = index;
	  p.currOp[2] = -1;
	  p.put(index, value);
  }

/***********************************************************
 * Hilfsfunktionen fuer Algorithmen
 **********************************************************/

/**
 * Vergleicht die Werte an 2 Indexstellen
 *
 *@param i1 Index fuer Vergleichswert1
 *@param i2 Index fuer Vergleichswert2
 *@throws InterruptedException
 *@return int gibt 1 zurueck, wenn der Wert an Index 1 kleiner ist als an Index 2,
 * bei Gleichheit 0 und wenn Index2-Wert < Index1-Wert dann -1
 */
protected int comp( int i1, int i2 ) throws InterruptedException {

	/*
	  p.currOp[0] = p.COMP;
	  p.currOp[1] = i1;
	  p.currOp[2] = i2;

	  int v1 = p.get(i1);
	  int v2 = p.get(i2);
	  */

	  int v1 = get(i1);
	  int v2 = get(i2);

	  return compVal(v1,v2);
  }

/**
 * Vergleicht 2 Werte miteinander
 *
 *@param v1 Vergleichswert1
 *@param v2 Vergleichswert2
 *@throws InterruptedException
 *@return int Gibt 1 zurueck, wenn Wert1 kleiner ist als Wert 2,
 * bei Gleichheit 0 und wenn Wert2 < Wert1 dann -1
 */
protected int compVal( int v1, int v2 ) throws InterruptedException {

	  p.opCount[p.COMP]++;

	  if (v1<v2) return -1;
	  else if (v1==v2) return 0;
	  else return 1;
  }


/**
 * Vertauscht 2 Werte innerhalb des Arrays miteinander
 *
 *@param i1 Der erste Index, dessen Wert an Index 2 geschrieben werden soll
 *@param i2 Der zweite Index, dessen Wert an Index 1 geschrieben werden soll
 *@throws InterruptedException
 *@return void
 */
protected void swap( int i1, int i2 ) throws InterruptedException {
/*
	  p.currOp[0] = p.SWAP;
	  p.currOp[1] = i1;
	  p.currOp[2] = i2;

	  int t = p.get(i1);
	  p.put(i1, p.get(i2));
	  p.put(i2, t);
	  */

	  int t = get(i1);
	  put(i1, get(i2) );
	  put(i2, t);

	  p.opCount[p.SWAP]++;
  }


}

